@file:JvmName("_AutocloseKt")
@file:OptIn(ExperimentalContracts::class)

package ru.landgrafhomyak.db.jdbc_kotlin_extensions

import org.intellij.lang.annotations.Language
import java.sql.Connection
import java.sql.PreparedStatement
import java.sql.ResultSet
import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.InvocationKind
import kotlin.contracts.contract
import ru.landgrafhomyak.utility.highlevel_try_finally.safeAutoClose1
import ru.landgrafhomyak.utility.highlevel_try_finally.safeAutoClose2

public inline fun <R> Connection.transaction(t: (Connection) -> R): R {
	contract {
		callsInPlace(t, InvocationKind.EXACTLY_ONCE)
	}

	this.executeStatement("BEGIN TRANSACTION")
	return safeAutoClose2(
		onError = { this.executeStatement("ROLLBACK TRANSACTION") },
		onSuccess = { this.executeStatement("COMMIT TRANSACTION") },
		action = { t(this) }
	)
}

public fun Connection.executeStatement(@Language("SQL") sql: String) {
	this.prepareStatement(sql) { ps -> ps.execute() }
}

public inline fun <R> Connection.prepareStatement(@Language("SQL") sql: String, action: (PreparedStatement) -> R): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
	}
	@Suppress("SqlSourceToSinkFlow")
	val ps = this.prepareStatement(sql)
	return safeAutoClose1(finally = ps::close) { action(ps) }
}

public inline fun <R> PreparedStatement.executeQuery(action: (ResultSet) -> R): R {
	contract {
		callsInPlace(action, InvocationKind.EXACTLY_ONCE)
	}
	return this.executeQuery().use(action)
}
